//=============================================================================
//
// class UserManager
//
// Author:			Grzegorz Lewandowski
// Created:			12/01/98
//
// Modified:		02/10/99
// By:				Grzegorz Lewandowski
// Changes:			class name changed from PasswordManager to UserManager
//
//=============================================================================

package webwisdom.tango.config;

import javax.swing.*;
import javax.swing.table.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

/**Main class for application managing users for Tango server
 * @version 	0.9, 98/12/01
 */
public class UserManager implements ActionListener
{
	private webwisdom.tango.server.PasswdManager passwordManager;
	private JFrame frame;
	private JTable table;
	
	private File lastDir;
	private boolean changesMade;


	/**Constructs new UserManager object with specified file as a parameter.
	 * @param userInfoFile file containing user information
	 */
	public UserManager( String userInfoFileName ) throws IOException
	{
		// initialize log
		// System.out.print( "initializing log..." );
		webwisdom.tango.test.Log.init( 10 );
		// System.out.println( " DONE" );
		
		// create PasswdManager object
		// System.out.print( "creating password manager..." );
		passwordManager = new webwisdom.tango.server.PasswdManager( userInfoFileName );
		// System.out.println( " DONE" );

		// create GUI
		// System.out.println( "creating GUI..." );
        frame = new JFrame( "Tango Server User Manager - " + userInfoFileName );

		//  menus
		JMenuBar menuBar;
		JMenu menu;
		JMenuItem menuItem;

		// System.out.println( " creating menus..." );
		menuBar = new JMenuBar();
		frame.setJMenuBar( menuBar );

		// System.out.print( "  creating File menu..." );
		menu = new JMenu( "File" );
		menuBar.add( menu );
		
		menuItem = new JMenuItem( "Open User File..." );
		menuItem.setActionCommand( "openFile" );
		menuItem.addActionListener( this );
		menu.add( menuItem );
		menuItem = new JMenuItem( "Save User File" );
		menuItem.setActionCommand( "saveFile" );
		menuItem.addActionListener( this );
		menu.add( menuItem );
		menuItem = new JMenuItem( "Exit" );
		menuItem.setActionCommand( "exit" );
		menuItem.addActionListener( this );
		menu.add( menuItem );
		// System.out.println( " DONE" );

		// System.out.print( "  creating User menu..." );
		menu = new JMenu( "User" );
		menuBar.add( menu );

		menuItem = new JMenuItem( "New User..." );
		menuItem.setActionCommand( "createNewUser" );
		menuItem.addActionListener( this );
		menu.add( menuItem );

		menuItem = new JMenuItem( "Delete User" );
		menuItem.setActionCommand( "deleteUser" );
		menuItem.addActionListener( this );
		menu.add( menuItem );
		
		menuItem = new JMenuItem( "Properties..." );
		menuItem.addActionListener( this );
		menu.add( menuItem );
		// System.out.println( " DONE" );

		// table of users
		// System.out.println( " creating table..." );
		Object[][] data = {
			{ "", "", "", "" } };
		Enumeration users, fullNames, statuses, eMails, passwords;
		Hashtable h = passwordManager.getStatusEntries();
		Hashtable f = passwordManager.getFullNameEntries();
		Hashtable e = passwordManager.getEMailEntries();
		int i = 0;
		if( h != null )
		{
			data = new Object[ h.size() ][];
			users = h.keys();
			fullNames = f.elements();
			statuses = h.elements();
			eMails = e.elements();
		
			// System.out.print( "  filling table with user information..." );
			while( users.hasMoreElements() )
			{
				data[ i ] = new Object[ 4 ];
				data[ i ][ 0 ] = users.nextElement();
				data[ i ][ 1 ] = fullNames.nextElement();
				data[ i ][ 2 ] = statuses.nextElement();
				data[ i ][ 3 ] = eMails.nextElement();
				i++;
			}
			// System.out.println( " DONE" );
		}

		TableModel model = new PasswordTableModel( data );
		table = new JTable( model );
		
		table.setPreferredScrollableViewportSize( new Dimension( 500, Math.min( 20 * i, 600 ) ) );
		table.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		
		table.addMouseListener( new MouseAdapter()
			{
				public void mouseClicked( MouseEvent e )
				{
					if( e.isMetaDown() )
					{
						int row = table.rowAtPoint( e.getPoint() );
						System.out.println( "right click at row " + row );
					}
					if( e.getClickCount() == 2 )
					{
						int row = table.rowAtPoint( e.getPoint() );
						try
						{
							editProperties( row );
						}
						catch( IOException ie )
						{
							JOptionPane.showMessageDialog( frame, "Unable to modify password file. " + ie.getMessage(), "User Properties", JOptionPane.ERROR_MESSAGE );
						}
					}
				}
			}
								);
		table.registerKeyboardAction( (ActionListener)this, "deleteUser", KeyStroke.getKeyStroke( java.awt.event.KeyEvent.VK_DELETE, 0 ), JComponent.WHEN_FOCUSED );

		// System.out.println( " creating scroller..." );
		//  create the scroll pane and add the table to it
		JScrollPane scrollPane = new JScrollPane( table );

		//  add the scroll pane to this window
		frame.getContentPane().add( scrollPane, BorderLayout.CENTER );

		// System.out.println( " adding listeners..." );
		frame.addWindowListener( new WindowAdapter()
		{
            public void windowClosing( WindowEvent e )
			{
				try
				{
					if( closeUserInfoFile() )
					{
						System.exit( 0 );
					}
				}
				catch( IOException ex )
				{
				}
            }
        } );
		
		// init last directory store
		lastDir = null;
		
		changesMade = false;
		
        frame.pack();
		frame.setVisible( true );
		// System.out.println( "GUI ready" );
    }

	//
	// main
	//
	public static void main( String[] args )
	{
        try
		{
            UIManager.setLookAndFeel( UIManager.getCrossPlatformLookAndFeelClassName() );
			// UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
        }
		catch( Exception e )
		{
			System.err.println( "WHAT?!!!" );
			System.err.println( "FATAL ERROR: " + e.getMessage() );
			return;
		}

		// create itself
		UserManager pm = null;
		String fileName = null;
		try
		{
			fileName = args[ 0 ];
		}
		catch( IndexOutOfBoundsException e )
		{
			// assume default file
			fileName = "users.pwd";
		}
		
		try
		{
			pm = new UserManager( fileName );
		}
		catch( IOException e )
		{
			System.err.println( "Unable to create UserManager. " + e.getMessage() );
		}
	}

	//
	// ActionListener
	//
	public void actionPerformed( ActionEvent e )
	{
		String command = e.getActionCommand();
		if( command.compareTo( "openFile" ) == 0 )
		{
			try
			{
				if( !closeUserInfoFile() )
				{
					return;
				}
			}
			catch( IOException ex )
			{
				System.err.println( "Unable to save user info file. " + ex.getMessage() );
				JOptionPane.showMessageDialog( frame, "Unable to save user info file. " + ex.getMessage(), "Tango User Manager", JOptionPane.ERROR_MESSAGE );
				return;
			}
			
			JFileChooser chooser = new JFileChooser();
			chooser.setFileSelectionMode( JFileChooser.FILES_ONLY );
			ExtensionFileFilter filter = new ExtensionFileFilter( ".pwd", "Tango Server User Files (*.pwd)" );
			chooser.addChoosableFileFilter( filter );
			chooser.setFileFilter( filter );
			if( lastDir != null )
			{
				chooser.setCurrentDirectory( lastDir );
			}
			
			int result = chooser.showOpenDialog( frame );
			if( result == JFileChooser.APPROVE_OPTION )
			{
				// store current directory for future use
				lastDir = chooser.getCurrentDirectory();
				try
				{
					openUserInfoFile( chooser.getSelectedFile().getPath() );
				}
				catch( IOException ex )
				{
					// System.err.println( "Unable to create PasswdManager. " + ex.getMessage() );
					JOptionPane.showMessageDialog( frame,
												   "Unable to open user information file. " + ex.getMessage(),
												   "Open File",
												   JOptionPane.ERROR_MESSAGE );
				}
			}			
		}
		else if( command.compareTo( "saveFile" ) == 0 )
		{
			try
			{
				passwordManager.update();
			}
			catch( IOException ex )
			{
				JOptionPane.showMessageDialog( frame,
											   "Unable to save user information file. " + ex.getMessage(),
											   "Save File", 
											   JOptionPane.ERROR_MESSAGE );
				return;
			}
			changesMade = false;
		}
		else if( command.compareTo( "exit" ) == 0 )
		{
			try
			{
				if( closeUserInfoFile() )
				{
					System.exit( 0 );
				}
			}
			catch( IOException ex )
			{
				System.out.println( "Unable to save user information file. " + ex.getMessage() );
				JOptionPane.showMessageDialog( frame, "Unable to save user information file. " + ex.getMessage(), "Exit", JOptionPane.ERROR_MESSAGE );
			}
		}
		else if( command.compareTo( "createNewUser" ) == 0 )
		{
			UserEditor ue = new UserEditor( frame, "Create New User" );
			if( ue.getResult() )
			{
				// modify data
				PasswordTableModel tm = (PasswordTableModel)table.getModel();
				Object[] userData = new Object[ 4 ];
				userData[ 0 ] = ue.getName();
				userData[ 1 ] = ue.getFullName();
				userData[ 2 ] = ue.getStatus();
				userData[ 3 ] = ue.getEMail();
				tm.addRow( userData );
			}
			
			try
			{
				passwordManager.addEntry( ue.getName(), ue.getPassword(), ue.getStatus(), ue.getFullName(), ue.getEMail() );
			}
			catch( Exception ex )
			{
				System.out.println( "Unable to modify user information. " + ex.getMessage() );
				JOptionPane.showMessageDialog( frame, "Unable to modify user information. " + ex.getMessage(), "Create New User", JOptionPane.ERROR_MESSAGE );
			}
			changesMade = true;
			frame.repaint();
		}
		else if( command.compareTo( "deleteUser" ) == 0 )
		{
			int n = table.getSelectedRow();
			try
			{
				// System.out.println( "removing user " + name );
				deleteUser( n );
			}
			catch( Exception ex )
			{
				// System.err.println( "Unable to modify password file. " + ex.getMessage() );
				JOptionPane.showMessageDialog( frame, "Unable to modify password file. " + ex.getMessage(), "Delete User", JOptionPane.ERROR_MESSAGE );
			}
		}
		else if( command.compareTo( "Properties..." ) == 0 )
		{
			int n = table.getSelectedRow();
			try
			{
				editProperties( n );
			}
			catch( Exception ex )
			{
				// System.err.println( "Unable to modify password file. " + ex.getMessage() );
				JOptionPane.showMessageDialog( frame, "Unable to modify password file. " + ex.getMessage(), "User Properties", JOptionPane.ERROR_MESSAGE );
			}
		}
		else
		{
			System.out.println( "actionPerformed( " + e + " )" );
		}
	}


	//
	// private methods
	//
	private void openUserInfoFile( String userInfoFileName ) throws IOException
	{
		// System.out.println( "You chose to open this file: " + chooser.getSelectedFile().getPath() );
		// System.out.println( "creating password manager..." );

		passwordManager = new webwisdom.tango.server.PasswdManager( userInfoFileName );
		// System.out.println( "password manager created" );

		// table of users
		// System.out.println( "creating table..." );
		Object[][] data = {
			{ "", "", "", "" } };
		Enumeration users, fullNames, statuses, eMails, passwords;
		Hashtable h = passwordManager.getStatusEntries();
		Hashtable f = passwordManager.getFullNameEntries();
		Hashtable m = passwordManager.getEMailEntries();

		if( h != null )
		{
			data = new Object[ h.size() ][];
			users = h.keys();
			fullNames = f.elements();
			statuses = h.elements();
			eMails = m.elements();

			// System.out.print( "  filling table with user information..." );
			int i = 0;
			while( users.hasMoreElements() )
			{
				data[ i ] = new Object[ 4 ];
				data[ i ][ 0 ] = users.nextElement();
				data[ i ][ 1 ] = fullNames.nextElement();
				data[ i ][ 2 ] = statuses.nextElement();
				data[ i ][ 3 ] = eMails.nextElement();
				i++;
			}
			// System.out.println( " DONE" );				
		}

		PasswordTableModel ptm = (PasswordTableModel)table.getModel();
		ptm.recreate( data );
		
		table.setPreferredScrollableViewportSize( new Dimension( 500, Math.min( 20 * h.size(), 600 ) ) );
		
		frame.setTitle( "Tango Server User Manager - " + userInfoFileName );
		changesMade = false;
		frame.pack();
		frame.repaint();	
	}
	
	private boolean closeUserInfoFile() throws IOException
	{
		if( changesMade )
		{
			// ask for save
			int result = JOptionPane.showConfirmDialog( frame,
														"Do you want to save the changes?",
														"Tango User Manager",
														JOptionPane.YES_NO_CANCEL_OPTION );
			if( result == JOptionPane.YES_OPTION )
			{
				// save and quit
				passwordManager.update();
				return true;
			}
			else if( result == JOptionPane.NO_OPTION )
			{
				// just quit
				return true;
			}
			else
			{
				// abort
				return false;
			}
		}
		
		return true;
	}

	private void editProperties( int n ) throws IOException
	{
		TableModel tm = table.getModel();
		String name = (String)tm.getValueAt( n, 0 );
		String fullName = (String)tm.getValueAt( n, 1 );
		String status = (String)tm.getValueAt( n, 2 );
		String eMail = (String)tm.getValueAt( n, 3 );
		UserEditor ue = new UserEditor( frame, "Edit User Properties", name, status, fullName, eMail );
		if( ue.getResult() )
		{
			// modify data
			tm.setValueAt( ue.getName(), n, 0 );
			tm.setValueAt( ue.getFullName(), n, 1 );
			tm.setValueAt( ue.getStatus(), n, 2 );
			tm.setValueAt( ue.getEMail(), n, 3 );
			changesMade = true;
		}
		// write to password file
		passwordManager.addEntry( ue.getName(), ue.getPassword(), ue.getStatus(), ue.getFullName(), ue.getEMail() );
		frame.repaint();
	}
	
	private void deleteUser( int n ) throws IOException
	{
		PasswordTableModel tm = (PasswordTableModel)table.getModel();
		String name = (String)tm.getValueAt( n, 0 );
		passwordManager.removeEntry( name );
		tm.removeRow( n );
		changesMade = true;
		frame.repaint();
	}
}
